@@ -209,6 +209,7 @@ class Generator:
         "_leading_comma",
         "_max_text_width",
         "_comments",
+        "_cache",
     )
 
     def __init__(
@@ -265,19 +266,28 @@ class Generator:
         self._leading_comma = leading_comma
         self._max_text_width = max_text_width
         self._comments = comments
+        self._cache = None
 
-    def generate(self, expression: t.Optional[exp.Expression]) -> str:
+    def generate(
+        self,
+        expression: t.Optional[exp.Expression],
+        cache: t.Optional[t.Dict[int, str]] = None,
+    ) -> str:
         """
         Generates a SQL string by interpreting the given syntax tree.
 
         Args
             expression: the syntax tree.
+            cache: an optional sql string cache. this leverages the hash of an expression which is slow, so only use this if you set _hash on each node.
 
         Returns
             the SQL string.
         """
+        if cache is not None:
+            self._cache = cache
         self.unsupported_messages = []
         sql = self.sql(expression).strip()
+        self._cache = None
 
         if self.unsupported_level == ErrorLevel.IGNORE:
             return sql
@@ -387,6 +397,12 @@ class Generator:
         if key:
             return self.sql(expression.args.get(key))
 
+        if self._cache is not None:
+            expression_id = hash(expression)
+
+            if expression_id in self._cache:
+                return self._cache[expression_id]
+
         transform = self.TRANSFORMS.get(expression.__class__)
 
         if callable(transform):
@@ -407,7 +423,11 @@ class Generator:
         else:
             raise ValueError(f"Expected an Expression. Received {type(expression)}: {expression}")
 
-        return self.maybe_comment(sql, expression) if self._comments and comment else sql
+        sql = self.maybe_comment(sql, expression) if self._comments and comment else sql
+
+        if self._cache is not None:
+            self._cache[expression_id] = sql
+        return sql
 
     def uncache_sql(self, expression: exp.Uncache) -> str:
         table = self.sql(expression, "this")
@@ -733,9 +753,9 @@ class Generator:
 
     def identifier_sql(self, expression: exp.Identifier) -> str:
         text = expression.name
-        text = text.lower() if self.normalize else text
+        text = text.lower() if self.normalize and not expression.quoted else text
         text = text.replace(self.identifier_end, self._escaped_identifier_end)
-        if expression.args.get("quoted") or should_identify(text, self.identify):
+        if expression.quoted or should_identify(text, self.identify):
             text = f"{self.identifier_start}{text}{self.identifier_end}"
         return text
 
@@ -1191,6 +1211,9 @@ class Generator:
         )
         return f"SET{expressions}"
 
+    def pragma_sql(self, expression: exp.Pragma) -> str:
+        return f"PRAGMA {self.sql(expression, 'this')}"
+
     def lock_sql(self, expression: exp.Lock) -> str:
         if self.LOCKING_READS_SUPPORTED:
             lock_type = "UPDATE" if expression.args["update"] else "SHARE"
@@ -1323,6 +1346,8 @@ class Generator:
         )
 
     def select_sql(self, expression: exp.Select) -> str:
+        kind = expression.args.get("kind")
+        kind = f" AS {kind}" if kind else ""
         hint = self.sql(expression, "hint")
         distinct = self.sql(expression, "distinct")
         distinct = f" {distinct}" if distinct else ""
@@ -1330,7 +1355,7 @@ class Generator:
         expressions = f"{self.sep()}{expressions}" if expressions else expressions
         sql = self.query_modifiers(
             expression,
-            f"SELECT{hint}{distinct}{expressions}",
+            f"SELECT{kind}{hint}{distinct}{expressions}",
             self.sql(expression, "into", comment=False),
             self.sql(expression, "from", comment=False),
         )
